global data_report_configs
import os
import sys
import jsonpickle
import models
from flask import Flask, request,send_from_directory, render_template, Blueprint
import json

api = Blueprint('scapinspector_api', __name__, template_folder='templates')

from sqlalchemy.ext.declarative import DeclarativeMeta
class AlchemyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj.__class__, DeclarativeMeta):
            # an SQLAlchemy class
            fields = {}
            for field in [x for x in dir(obj) if not x.startswith('_') and x != 'metadata']:
                data = obj.__getattribute__(field)
                try:
                    json.dumps(data) # this will fail on non-encodable values, like other classes
                    fields[field] = data
                except TypeError:
                    fields[field] = None
            # a json-encodable dict
            return fields

        return json.JSONEncoder.default(self, obj)


@api.route('/api/scapinspector/get_benchmarks', methods=['POST'])
def config():
    """Return loaded benchmarks """
    benchmarks=models.get_benchmarks()
    
    json_data = json.dumps(benchmarks, cls=AlchemyEncoder)
    #json_data = jsonpickle.encode(benchmarks, unpicklable=False)
    return json.dumps([dict(r) for r in benchmarks])





import os
from scapinspector.api import api
from scapinspector.api_static import api as api_static

from flask import Flask

app = Flask(__name__, static_url_path='')
app.register_blueprint(api)
app.register_blueprint(api_static)


if __name__ == "__main__":
    app.run(host="0.0.0.0")



import os
import sys
import json
import untangle
import map_xml
import benchmarks
import profile_core
import load
import db

# this function loads an SCAP datastream file
# the file may contain many parts, multiple benchmarks, 
# profiles, definitions, dictionaries and other components
# It then parses it for relivant info needed for the UI 
def parse_datastream(xccdf_data_stream_file):
    print ("Parsing:",xccdf_data_stream_file)
    with open(xccdf_data_stream_file, 'r') as ds:
        data=ds.read()
    try:
        result_meta = untangle.parse(data)
    except Exception as err:
        sys.exit("There is an issue the data stream:{} ".format(err))
    #package=map_xml.parse(result_meta,{})
    #namespaces=package['namespace']
    #print(namespaces)

    parsed_data=benchmarks.get(result_meta,xccdf_data_stream_file)
    return parsed_data




import models
import db

def  into_database(data):
    dbc=db.get_db()
    session=dbc['session']

    # all data parsed. lets insert
    print("Loading into db..")
    db_rule_list={}

    for rule in data['rules']:
        #print(rule)
        r=models.Rule( ref_id=rule['id'],
                        title=rule['title'],
                        description=rule['description'])
        db_rule_list[r.ref_id]=r
        session.add(r)
    session.commit()   

    for benchmark in data['benchmarks']:
        b=models.Benchmark(ref_id     =benchmark['id'],
                            title      =benchmark['title'],
                            description=benchmark['description'],
                            file       =benchmark['file'])
        session.add(b)
    
        for profile in benchmark['profiles']:
            p=models.Profile(  ref_id     =profile['id'],
                                title      =profile['title'],
                                description=profile['description'],
                                benchmark  =b)
            session.add(p)
            
            for select  in profile['selects']:
                s=models.Select(   ref_id   =select['ref_id'],
                                    selected =select['selected'],
                                    profile  =p,
                                    rule     =db_rule_list[select['ref_id']])
                session.add(s)
        
        #recursive nested group builder....
        add_groups(benchmark,b,db_rule_list,session)     
        session.commit()   



def add_groups(node,db_model_benchmark,db_rules,session, group_parent=None):
    for group in node['groups']:

        g=models.Group(
                ref_id       = group['id'],
                title        = group['title'],
                description  = group['description'],
                parent       = group_parent,
                benchmark    = db_model_benchmark)
        session.add(g)

        for rule in group['rules']:
            r=models.Group_Rule(   ref_id = rule,
                                    rule   = db_rules[rule],
                                    parent = g)
            session.add(r)
        
        add_groups(group,db_model_benchmark,db_rules,session,g)



import click
import os
import db
import parse_xccd_ds
import load
import models
import ConfigParser
from api import api
from api_static import api as api_static
from flask import Flask

config_file = os.path.expanduser('~/.scapinspector.cfg')
    
@click.group(chain=False)
def main():
    """
    Parse SCAP datastreams, result content and tailor custom profiles
    """
    pass

@main.command("init-db")
@click.option("--host")#, env_var="SCAPINSPECTOR_HOST")
@click.option("--user")#, env_var="SCAPINSPECTOR_USER")
@click.password_option() #, env_var="SCAPINSPECTOR_PASSWORD")
@click.option("--database") #, env_var="SCAPINSPECTOR_DB")
@click.option("--port") #, env_var="SCAPINSPECTOR_DB")
def init(host,user,password,database,port):
    """
    Initialize the Mysql storage db for data
    """
    db.set_mysql_config(host,user,password,database,port)
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.add_section('mariadb')
    config.set('mariadb', 'host'    , host)
    config.set('mariadb', 'user'    , user)
    config.set('mariadb', 'password', password)
    config.set('mariadb', 'database', database)
    config.set('mariadb', 'port'    , port)
    with open(config_file, 'wb') as configfile:
        config.write(configfile)
    models.build(True)
    pass

@main.command("show-config")
def show_config():
    """
    Show the configured db values
    """
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.read(config_file)
    host     = "localhost"
    user     = "root"
    password = ""
    database = ""
    port     = "3306"
    try:
        host     = config.get('mariadb', 'host')
        user     = config.get('mariadb', 'user')
        database = config.get('mariadb', 'database')
        password = config.get('mariadb', 'password')
        port     = config.get('mariadb', 'port')
    except Exception as ex:
        print("Configuration Error", ex)
    print("""
    Mariadb settings
      Host:     {0}
      Port:     {4}
      User:     {1}
      Password: {2}
      Database: {3}
    """.format(host,user,password,database,port))

    pass


def load_config():
    """
    Show the configured db values
    """
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.read(config_file)
    try:
        host     = config.get('mariadb', 'host')
        user     = config.get('mariadb', 'user')
        database = config.get('mariadb', 'database')
        password = config.get('mariadb', 'password')
        port     = config.get('mariadb', 'port')
        db.set_mysql_config(host,user,password,database,port)
    except Exception as ex:
        print("Configuration Error", ex)



@main.command("load-db")
def load_data():
    """
    Load a directory of scap datastreams into a custom database
    """
    directory="/usr/share/xml/scap/ssg/content/"
    load_config()
    session=db.get_db()
    if None == session: 
        print ("Init DB failed")
        exit(1)

    for dirname,dirnames, filenames in os.walk(directory):
        print("Dir:",dirname)
        for filename in filenames:
            print("File:",filename)
            if filename.endswith("-ds.xml"): 
                data=parse_xccd_ds.parse_datastream(directory+filename)
                #print (data)
                load.into_database(data)
                continue
            else:
                continue    
    pass


@main.command("ui")
def webserver():
    """
    run webserver
    """
    load_config()

    app = Flask(__name__, static_url_path='')
    app.register_blueprint(api)
    app.register_blueprint(api_static)
    app.run(host="0.0.0.0")


if __name__ == "__main__":
    main()


import profile_core

def parse(parent,namespace,xnspath=""):
    """Recursively parse an xml tree, and map each tag and all possible attrributes"""
    elements=[]
    if None is namespace:
        namespace={}
    #elements.append(type_obj(parent))
    for child in parent.get_elements():
        for attrib_name in child._attributes:
            if "xmlns:" in attrib_name:
                namespace[xnspath+"."+attrib_name[6:]]=child[attrib_name]
          
        elements.append(profile_core.type_obj(child,namespace))
        ns=profile_core.get_ns(child._name)
        results= parse(child,namespace,ns)
        
        for k,v in results['namespace'].items():
            namespace[k]=v

        if len(results['elements']) > 0:
            for el in results['elements']:
                 elements.append(el)
            elements=profile_core.merge_types(elements,namespace)

    return {'elements':elements,'namespace':namespace}


def get_namespaces(parent):
    """parse the first element of a datastream xml tree for namespaces"""
    namespace={}
    for child in parent.get_elements():
        for attrib_name in child._attributes:
            if "xmlns:" in attrib_name:
                namespace[attrib_name[6:]]=child[attrib_name]

    print(namespace)
    return namespace



import profile_core

def get_by_id(id,root):
    node=profile_core.get_node_by_attribute('id',id,root)
    if None==node:
        return None
    rule={}
    #print(node)
    title              = profile_core.node_values(node,"title"       )
    description        = profile_core.node_values(node,"description" )
    #title              = "sam"
    #description        = "bob"
    rule['id']         = id
    rule['title']      = title
    rule['description']= description
    return rule


def get_at_this_level(root):
    # TODO manage for reference
    nodes=root.get_elements()
    if None==nodes:
        return []
    rules_list=[]
    for node in nodes:
        tag=profile_core.get_tag_and_namespace(node._name)

        if tag['name'] == 'Rule':
            rule={}
            node_id            = node['id']
            if None == node_id:
                node_id=node['ref_id']
            if None == node_id:
                node_id=None

            if node_id == None:
                print ("DidntFind Rule by Ref")

            title              = profile_core.node_values(node,"title"       )
            description        = profile_core.node_values(node,"description" )
            rule['id']         = node_id
            rule['title']      = title
            rule['description']= description
            rules_list.append(rule)
    return rules_list


from flask import Blueprint,render_template,send_from_directory

api = Blueprint('scapinspector_api_static', __name__, template_folder='templates')


@api.route('/')
def index():
    return render_template('index.html')

#Non api related calls
@api.route('/node_modules/<path:path>')
def send_js(path):
    return send_from_directory('node_modules', path)



import sys
import json
import untangle

def merge_dict(d1,d2):
    """
    Given any number of dicts, shallow copy and merge into a new dict,
    precedence goes to key value pairs in latter dicts.
    """
    if d2 != {}:
        match=False
        for d2k, d2v in d2.iteritems():
            for d1k, d1v in d1.iteritems():
                if d1v == d2v:
                    match=True
                    break
            if False == match:
                #print (dest_item +" "+d2[dest_item])
                d1.update(d2k,d1v)
    return d1

def merge_dicts(*dict_args):
    """
    Given any number of dicts, shallow copy and merge into a new dict,
    precedence goes to key value pairs in latter dicts.
    """
    result = {}
    for dictionary in dict_args:
        result.update(dictionary)
    return result

def merge_types(items):
    master=[]
    if None == items:
        return []
    for item in items:
        found=False
        for m,mitem in enumerate(master):
            #print (m,mitem)
            if mitem["name"] == item["name"]:
                temp_attrib=[]
                for a in mitem["attributes"]:
                    temp_attrib.append(a)
                for a in item["attributes"]:
                    temp_attrib.append(a)
                master[m]["attributes"]=f5(temp_attrib)
                found=True
                break
        if True!=found:
                master.append(item)
    return master


def f5(seq, idfun=None): 
   # order preserving
   if idfun is None:
       def idfun(x): return x
   seen = {}
   result = []
   for item in seq:
       marker = idfun(item)
       # in old Python versions:
       # if seen.has_key(marker)
       # but in new ones:
       if marker in seen: continue
       seen[marker] = 1
       result.append(item)
   return result

def get_ns(tag_name):
    ns=""
    index=tag_name.find('_')
    if index == -1:
        return ""
    #name=tag_name[index+1:]
    ns=tag_name[:index]

    return ns


def type_obj(item):
    attributes=[]
    for attrib in item._attributes:
        attributes.append(attrib)

    ns=get_ns(item._name)

    index=item._name.find('_')
    name=item._name[index+1:]
    
    return {'name':name,'attributes':attributes,'namespace':ns}

def get_tag_and_namespace(text):
    index=text.find('_')
    
    ns=""
    name=text[index+1:]
    ns=text[:index]

    return {'name':name,'namespace':ns}

def print_tree_tab(depth=0):
    pad=""
    if 0!= depth:
        for _ in range(depth):
            pad+="-"
    pad+="|"
    print(pad,depth)

def nodes(element,path,depth=0,debug=None):
    if None != debug:
        print_tree_tab(depth)
        
    elements=[]
    index=path.find('/')
    if -1 == index:
        name=path
        sub_path=None
    else:
        name=path[:index]
        sub_path=path[path.find('/')+1:]
    
    try:
        for item in element.get_elements():
            tag=get_tag_and_namespace(item._name)
            #print("--",tag['name'],name)    
            if name == tag['name']:
                if None != debug:
                    print("Found: ",name)    
                if None != sub_path:
                    results=nodes(item,sub_path,depth+1,debug)
                    for result in results:
                        elements.append(result)
                else:
                    elements.append(item)
        # if this is the end, return a single element or an array based on array depth
        if depth==0 and len(elements)==1:
            return elements[0]
        if depth==0 and len(elements)==0:
            return []
        # not the end. return all results
        return elements
        #search children
    except Exception as ex:
        print ("Error:",path)





def node_values(root,path):
    results = nodes(root,path)
    if None == results:
        return None
    else:
        return results.cdata

def get_node_by_attribute(attribute_name,attribute_value,root,depth=0):
    #if depth==0:
    #    print(attribute_name,attribute_value)
    try:        
        for node in root.get_elements():
            #print ("Looking for ",attribute_name,attribute_value)
            if None !=node[attribute_name]:
                #print ("found attribute",attribute_name)
                #print ("found attribute",attribute_name,"need",attribute_value,"have",node[attribute_name])
                if node[attribute_name] == attribute_value:
                    #print ("see attribute",attribute_value)
                    #print(attribute_name,node)
                    return node
            results=get_node_by_attribute(attribute_name,attribute_value,node,depth+1)
            if None!=results:
                return results
            if depth==0:
                print ("Didn't Find "+attribute_name)
        return None
    except Exception as err:
        print("----------------")
        print(root)
        print("----------------")
        print(err)
        return None


def get_id_or_ref(node,root):
    if None != node['id']:
        #print (" id found",node['id'])
        return  node['id']
    if None != node['idref']:
        ref_id = node['idref']
        #
        # print ("ref id found",ref_id)
        node=get_node_by_attribute('id',ref_id,root)
        if None !=node:
            return node.cdata
        return node
    print ("nothing found",ref_id)




global mysql_config
import os
from sqlalchemy import create_engine
from sqlalchemy import Column, String, Integer, ForeignKey,DateTime
from sqlalchemy.sql import func
from sqlalchemy.orm import relationship
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
import logging
import click
from flask import current_app, g
from flask.cli import with_appcontext

#logging.basicConfig()
#logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)

db_conn=None
mysql_config=None

def set_mysql_config(host,user,password,database,port):
    global mysql_config
    mysql_config={}
    mysql_config['user']    = user
    mysql_config['password']= password
    mysql_config['host']    = host
    mysql_config['database']= database
    mysql_config['port']    = port



def get_mysql_config():
    global mysql_config
    config={}
    if mysql_config==None:
        mysql_config={}

    if 'user' not in  mysql_config:
        config['user']     = os.environ.get('SCAPINSPECTOR_DB_USER') 
    else:
        config['user']=mysql_config['user']

    if 'password' not in mysql_config:
        config['password'] = os.environ.get('SCAPINSPECTORT_DB_PASS')
    else:
        config['password']=mysql_config['password']

    if 'host' not in mysql_config:
        config['host']     = os.environ.get('SCAPINSPECTOR_DB_HOST') 
    else:
        config['host']=mysql_config['host']

    if 'database' not in mysql_config:
        config['database'] = os.environ.get('SCAPINSPECTOR_DB_NAME') 
    else:
        config['database']=mysql_config['database']

    if 'port' not in  mysql_config:
        config['port']     = os.environ.get('SCAPINSPECTOR_PORT') 
    else:
        config['port']=mysql_config['port']
    
    if None == config['port']:
        config['port']='3306'
    return config



def connect():

    mysql_config=get_mysql_config()

    if  None == mysql_config :
        connect_string = "mysql+pymysql://{}@{}:{}/{}?charset=utf8mb4".format( mysql_config['user'],
                                                                            mysql_config['host'],
                                                                            mysql_config['port'],
                                                                            mysql_config['database']
                                                                            )
    else:    
        connect_string = "mysql+pymysql://{}:{}@{}:{}/{}?charset=utf8mb4".format( mysql_config['user'],
                                                                            mysql_config['password'],
                                                                            mysql_config['host'],
                                                                            mysql_config['port'],
                                                                            mysql_config['database']
                                                                            )


    #print (connect_string)
    engine = create_engine(connect_string, convert_unicode=True, echo=False)
    Session = sessionmaker(bind=engine)
    session=Session()
    return {'session':session,'engine':engine}



def get_db():
    global db_conn
    try:
        if 'db' not in g:
            g.db = connect()
        return g.db
    except :
        db_conn=connect()
        return db_conn





def close_db(e=None):
    db = g.pop('db', None)
    if db is not None:
        db.close()




from setuptools import setup


setup(
    name='scapinspector',
    version='1.0.6',
    packages=['scapinspector',],
    include_package_data=True,
    url='https://github.com/chris17453/scapinspector/',
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description=open('README.md').read(),
    summary= 'SCAP tool for parsing scap datastreams, creating tailoring files and parsing report results',
    author= 'Charles Watkins',
    author_email= 'charles@titandws.com',
    description= 'SCAP tool for parsing scap datastreams, creating tailoring files and parsing report results',
    platform= 'All',    
    install_requires=[
        'flask',
        'sqlalchemy',
        'pyyaml',
        'pymysql',
        'jsonpickle',
    ],

)






import profile_core
import rules
from sqlalchemy.orm import sessionmaker
from sqlalchemy import create_engine


# benchmarks
#   - benchmark
#       title
#       id
#       desc
#       profiles:
#         - profile 
#             title          
#             description
#             id
#             selects
#               - select
#                   idref
#                   selected
#       groups
#         - group
#             title          
#             description
#             id
#             rules
#             - groups
#
# rules
#   - rule
#     id 
#     title
#     description



def get(root,file):
    benchmark_nodes=profile_core.nodes(root,"data_stream_collection/component/Benchmark")
    benchmark_list=[]
    rules_list={}
    profile_count=0
    load_profiles=True
    #  Main loop for bulding profile catalog
    for benchmark_node in benchmark_nodes:
        benchmark_id           = benchmark_node["id"]
        profile_nodes          = profile_core.nodes      (benchmark_node,"Profile"     )
        benchmark_title        = profile_core.node_values(benchmark_node,"title"       )
        benchmark_description  = profile_core.node_values(benchmark_node,"description" )
        benchmark_profiles     = []


        if load_profiles is True:
            for profile in profile_nodes:
                profile_count+=1
                profile_id           = profile_core.get_id_or_ref(profile,root)
                profile_select_nodes = profile_core.nodes      (profile,"select"      )
                profile_title        = profile_core.node_values(profile,"title"       )
                profile_description  = profile_core.node_values(profile,"description" )
                
                selects=[]
                for select_node in profile_select_nodes:
                    select_id_ref     = select_node['idref']
                    select_selected   = select_node['selected']
                    select                 = {}
                    select['ref_id']        = select_id_ref
                    select['selected']     = select_selected
                    #print(select)
                    if select_id_ref not in rules_list:
                        rules_list[select_id_ref]=rules.get_by_id(select_id_ref,root)
                    selects.append(select)
                    

                profile                        = {}
                profile['id']                  = profile_id
                profile['title']               = profile_title
                profile['description']         = profile_description
                profile['selects']             = selects
                benchmark_profiles.append(profile)
        
        benchmark_groups = group_get(benchmark_node,rules_list)

        for rule in benchmark_groups['rules']:
            if rule not in rules_list:
                rules_list[rule]=benchmark_groups['rules'][rule]

        benchmark                = {}
        benchmark['id']          = benchmark_id
        benchmark['file']        = file
        benchmark['title']       = benchmark_title
        benchmark['description'] = benchmark_description
        benchmark['profiles']    = benchmark_profiles
        benchmark['groups']      = benchmark_groups['groups']
        benchmark['group_rules'] = benchmark_groups['rules']

        benchmark_list.append(benchmark)

    
    # dict to array
    stripped_rules_list=[]
    for rule in rules_list:
        stripped_rules_list.append(rules_list[rule])

    benchmark_count=len(benchmark_list)
    rules_count=len(stripped_rules_list)
    return {'benchmarks':benchmark_list,'rules':stripped_rules_list,'benchmark_count':benchmark_count,'profile_count':profile_count,'rule_count':rules_count}

# TODO maybe load rule defs at end...
# TODO identify why I dont see so many rules... diff profile?
def group_get(root,rules_list):
    group_nodes = profile_core.nodes(root,"Group")

    groups=[]
    for group_node in group_nodes:
        group_id              = group_node['id']
        group_selected        = group_node['selected']
        group_title           = profile_core.node_values(group_node,"title" )
        group_description     = profile_core.node_values(group_node,"description" )
        group                 = {}
        group['id']           = group_id
        group['selected']     = group_selected
        group['title']        = group_title
        group['description']  = group_description
        
        local_rules=[]        
        group_rules=rules.get_at_this_level(group_node)
        #print(group_rules)
        for group_rule in group_rules:
            rule_id=group_rule['id']
            if None != rule_id and rule_id not in rules_list:
                rules_list[rule_id]=group_rule
            local_rules.append(rule_id)

        group['rules']        = local_rules
        
        groupsresults=group_get(group_node,rules_list)
        group['groups']       =groupsresults['groups']
        groupsresults_rule_list =groupsresults['rules']
        for rule in groupsresults_rule_list:
            if rule not in rules_list:
                rules_list[rule]=groupsresults_rule_list[rule]
        #
        #if group_id not in rules_list:
        #   rules_list[select_id_ref]=rules.get_by_id(select_id_ref,root,namespaces)
        groups.append(group)
    return {'groups':groups,'rules':rules_list}
            


import os
from sqlalchemy import create_engine
from sqlalchemy import Column, String, Integer, ForeignKey,DateTime ,DECIMAL
from sqlalchemy.sql import func
from sqlalchemy.orm import relationship
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
import logging
import db

Base = declarative_base()

class Version(Base):
    __tablename__ = 'version'
    id            = Column(Integer, primary_key=True)
    version       = Column(String(255))
    description   = Column(String(255))
    created       = Column(DateTime(timezone=True), default=func.now())

class Results(Base):
    __tablename__ = 'results'
    id            = Column(Integer,primary_key=True)
    server        = Column(String(255))
    group         = Column(String(255))
    profile       = Column(String(255))
    score         = Column(DECIMAL(18,4))
    high          = Column(DECIMAL(18,4))
    medium        = Column(DECIMAL(18,4))
    low           = Column(DECIMAL(18,4))
    unknown       = Column(DECIMAL(18,4))
    scanned       = Column(DateTime(timezone=True), default=func.now())
    

class Benchmark(Base):
    __tablename__ = 'benchmarks'
    id            = Column(Integer, primary_key=True)
    ref_id        = Column(String(255))
    title         = Column(String(2048))
    description   = Column(String(10000))
    file          = Column(String(2048))
    description   = Column(String(2048))
    created       = Column(DateTime(timezone=True), default=func.now())


class Profile(Base):
    __tablename__ = 'profiles'
    id            = Column(Integer, primary_key=True)
    ref_id        = Column(String(255))
    title         = Column(String(2048))
    file          = Column(String(2048))
    description   = Column(String(10000))
    benchmark_id  = Column(Integer, ForeignKey(Benchmark.id))
    benchmark     = relationship(Benchmark)
    created       = Column(DateTime(timezone=True), default=func.now())

class Rule(Base):
    __tablename__ = 'rules'
    id            = Column(Integer, primary_key=True)
    ref_id        = Column(String(255))
    title         = Column(String(2048))
    description   = Column(String(10000))
    created       = Column(DateTime(timezone=True), default=func.now())


class Group(Base):
    __tablename__ = 'groups'
    id            = Column(Integer, primary_key=True)
    ref_id        = Column(String(255))
    title         = Column(String(2048))
    description   = Column(String(10000))
    #rule_id = Column(Integer, ForeignKey(Rule.id))
    #rule=relationship(Rule)
    parent_id     = Column(Integer, ForeignKey('groups.id'))
    parent        = relationship('Group', remote_side=[id])
    benchmark_id  = Column(Integer, ForeignKey(Benchmark.id))
    benchmark     = relationship(Benchmark)
    created       = Column(DateTime(timezone=True), default=func.now())

class Group_Rule(Base):
    __tablename__ = 'group_rule'
    id            = Column(Integer, primary_key=True)
    ref_id        = Column(String(255))
    rule_id = Column(Integer, ForeignKey(Rule.id))
    rule=relationship(Rule)
    parent_id     = Column(Integer, ForeignKey('groups.id'))
    parent        = relationship(Group)
    created       = Column(DateTime(timezone=True), default=func.now())


class Select(Base):
    __tablename__ = 'select'
    id            = Column(Integer, primary_key=True)
    ref_id        = Column(String(255))
    selected      = Column(String(10))
    rule_id       = Column(Integer, ForeignKey(Rule.id))
    rule          = relationship(Rule)
    profile_id    = Column(Integer, ForeignKey(Profile.id))
    profile       = relationship(Profile)
    created       = Column(DateTime(timezone=True), default=func.now())

def get_benchmarks():
    dbc=db.get_db()
    session=dbc['session']
    results=session.query(Benchmark).order_by(Benchmark.id)
    return results

def build(drop=False):
    """Build the tables for the benchmark/xccdf model"""
    conn=db.get_db()
    engine=conn['engine']
    Base.metadata.reflect(engine)
    if True == drop:
        Base.metadata.drop_all(engine)
    Base.metadata.create_all(engine)
